#include <iostream>
#include <set>
#include <unordered_set>
#include <chrono>
#include <random>

template<typename Func>
long long measureTime(Func func) {
    auto start = std::chrono::high_resolution_clock::now();
    func();
    auto end = std::chrono::high_resolution_clock::now();
    return std::chrono::duration_cast<std::chrono::microseconds>(end - start).count();
}

int main() {
    const int NUM_ELEMENTS = 1000000;
    const int NUM_SEARCHES = 10000;

    std::set<int> orderedSet;
    std::unordered_set<int> unorderedSet;

    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(1, NUM_ELEMENTS);

    // Insertion
    auto insertOrdered = [&]() {
        for (int i = 0; i < NUM_ELEMENTS; ++i) {
            orderedSet.insert(dis(gen));
        }
    };
    auto insertUnordered = [&]() {
        for (int i = 0; i < NUM_ELEMENTS; ++i) {
            unorderedSet.insert(dis(gen));
        }
    };

    std::cout << "Insertion time (microseconds):" << std::endl;
    std::cout << "set: " << measureTime(insertOrdered) << std::endl;
    std::cout << "unordered_set: " << measureTime(insertUnordered) << std::endl;

    // Search
    auto searchOrdered = [&]() {
        for (int i = 0; i < NUM_SEARCHES; ++i) {
            orderedSet.find(dis(gen));
        }
    };
    auto searchUnordered = [&]() {
        for (int i = 0; i < NUM_SEARCHES; ++i) {
            unorderedSet.find(dis(gen));
        }
    };

    std::cout << "Search time (microseconds):" << std::endl;
    std::cout << "set: " << measureTime(searchOrdered) << std::endl;
    std::cout << "unordered_set: " << measureTime(searchUnordered) << std::endl;

    return 0;
}
